@@ -1,20 +1,22 @@
 # flake8: noqa
 
+# Define those first, so we can import them during library initialization
+__author__ = "DevilXD"
+__version__ = "0.2.0"
+
+
 from .cache import *
 from .enums import *
 from .items import *
 from .match import *
 from .stats import *
+from .bounty import *
+from .mixins import *
 from .player import *
 from .status import *
-from .mixins import *
 from .champion import *
 from .exceptions import *
 from .api import PaladinsAPI
 from .endpoint import Endpoint
 from .statuspage import StatusPage
 from .utils import Lookup, Duration
-
-
-__author__ = "DevilXD"
-__version__ = "0.1.7"
