@@ -2,17 +2,21 @@ from __future__ import annotations
 
 import logging
 from itertools import count
-from typing import Any, Optional, Union, List, Dict, Generator, TYPE_CHECKING
+from typing import Optional, Union, List, Dict, Iterable, Generator, TYPE_CHECKING
 
 from .exceptions import NotFound
-from .utils import _convert_map_name
-from .enums import Queue, Language, Region, Rank
-from .mixins import APIClient, CacheObject, MatchMixin, MatchPlayerMixin, Expandable, WinLoseMixin
+from .enums import Queue, Region, Rank
+from .utils import chunk, _convert_map_name, _deduplicate
+from .mixins import (
+    CacheClient, CacheObject, MatchMixin, MatchPlayerMixin, Expandable, WinLoseMixin
+)
 
 if TYPE_CHECKING:
-    from .api import PaladinsAPI
-    from .champion import Champion
-    from .player import PartialPlayer, Player  # noqa
+    from . import responses
+    from .enums import Language
+    from .champion import Champion, Skin
+    from .cache import DataCache, CacheEntry
+    from .player import PartialPlayer, Player
 
 
 __all__ = [
@@ -25,7 +29,24 @@ __all__ = [
 logger = logging.getLogger(__package__)
 
 
-class PartialMatch(MatchPlayerMixin, MatchMixin, Expandable):
+# this is a close duplicate of `PaladinsAPI.get_players`, modified for speed and its usage
+async def _get_players(cache: DataCache, player_ids: Iterable[int]) -> Dict[int, Player]:
+    ids_list: List[int] = _deduplicate(player_ids, 0)  # also remove private accounts
+    if not ids_list:  # pragma: no cover
+        return {}
+    from .player import Player  # cyclic import
+    players_dict: Dict[int, Player] = {}
+    for chunk_ids in chunk(ids_list, 20):
+        chunk_response = await cache.request("getplayerbatch", ','.join(map(str, chunk_ids)))
+        for player_data in chunk_response:
+            if player_data["ret_msg"]:  # pragma: no cover, skip private accounts
+                continue
+            player = Player(cache, player_data)
+            players_dict[player.id] = player
+    return players_dict
+
+
+class PartialMatch(MatchPlayerMixin, MatchMixin, Expandable["Match"]):
     """
     Represents a match from a single player's perspective only.
 
@@ -92,8 +113,9 @@ class PartialMatch(MatchPlayerMixin, MatchMixin, Expandable):
         The amount of objective time the player got, in seconds.
     multikill_max : int
         The maximum multikill player did during the match.
-    skin : CacheObject
-        The skin the player had equipped for this match.
+    skin : Union[Skin, CacheObject]
+        The skin the player had equipped for this match.\n
+        With incomplete cache, this will be a `CacheObject` with the name and ID set.
     team_number : Literal[1, 2]
         The team this player belongs to.
     team_score : int
@@ -102,9 +124,13 @@ class PartialMatch(MatchPlayerMixin, MatchMixin, Expandable):
         `True` if the player won this match, `False` otherwise.
     """
     def __init__(
-        self, player: Union["PartialPlayer", "Player"], language: Language, match_data: dict
+        self,
+        player: Union[PartialPlayer, Player],
+        language: Language,
+        cache_entry: Optional[CacheEntry],
+        match_data: responses.HistoryMatchObject,
     ):
-        MatchPlayerMixin.__init__(self, player, language, match_data)
+        MatchPlayerMixin.__init__(self, player, cache_entry, match_data)
         MatchMixin.__init__(self, match_data)
         self._language = language
 
@@ -128,7 +154,8 @@ class PartialMatch(MatchPlayerMixin, MatchMixin, Expandable):
         response = await self._api.request("getmatchdetails", self.id)
         if not response:
             raise NotFound("Match")
-        return Match(self._api, self._language, response, {})
+        cache_entry = self._api.get_entry(self._language)
+        return Match(self._api, cache_entry, response, {})
 
     def __repr__(self) -> str:
         return f"{self.queue.name}: {self.champion.name}: {self.kda_text}"
@@ -150,10 +177,19 @@ class MatchPlayer(MatchPlayerMixin):
 
     Attributes
     ----------
+    match : Match
+        The match this player belongs to.
     player : Union[PartialPlayer, Player]
-        The player who participated in this match.\n
+        The player itself who participated in this match.\n
         This is usually a new partial player object.\n
         All attributes, Name, ID and Platform, should be present.
+    rank : Optional[Rank]
+        The player's rank.
+
+        .. warning::
+
+            Due to API limitations, this is only available for matches played in ranked queues.\n
+            For other queues, this attribute will be `None`.
     champion : Union[Champion, CacheObject]
         The champion used by the player in this match.\n
         With incomplete cache, this will be a `CacheObject` with the name and ID set.
@@ -187,8 +223,9 @@ class MatchPlayer(MatchPlayerMixin):
         The amount of objective time the player got, in seconds.
     multikill_max : int
         The maximum multikill player did during the match.
-    skin : CacheObject
-        The skin the player had equipped for this match.
+    skin : Union[Skin, CacheObject]
+        The skin the player had equipped for this match.\n
+        With incomplete cache, this will be a `CacheObject` with the name and ID set.
     team_number : Literal[1, 2]
         The team this player belongs to.
     team_score : int
@@ -213,23 +250,28 @@ class MatchPlayer(MatchPlayerMixin):
     """
     def __init__(
         self,
-        api: "PaladinsAPI",
-        language: Language,
-        player_data: Dict[str, Any],
+        match: Match,
+        cache_entry: Optional[CacheEntry],
+        player_data: responses.MatchPlayerObject,
         parties: Dict[int, int],
         players: Dict[int, Player],
     ):
         player: Optional[Union[PartialPlayer, Player]] = players.get(int(player_data["playerId"]))
         if player is None:
             # if no full player was found
-            from .player import PartialPlayer  # noqa, cyclic imports
+            from .player import PartialPlayer  # cyclic imports
             player = PartialPlayer(
-                api,
+                match._api,
                 id=player_data["playerId"],
                 name=player_data["playerName"],
                 platform=player_data["playerPortalId"],
             )
-        super().__init__(player, language, player_data)
+        super().__init__(player, cache_entry, player_data)
+        self.rank: Optional[Rank]
+        if match.queue.is_ranked():
+            self.rank = Rank(player_data["League_Tier"], _return_default=True)
+        else:
+            self.rank = None
         self.points_captured: int = player_data["Kills_Gold_Fury"]
         self.push_successes: int = player_data["Kills_Fire_Giant"]
         self.kills_bot: int  = player_data["Kills_Bot"]
@@ -254,9 +296,10 @@ class MatchPlayer(MatchPlayerMixin):
         )
 
 
-class Match(APIClient, MatchMixin):
+class Match(CacheClient, MatchMixin):
     """
     Represents already-played full match information.
+
     You can get this from the `PaladinsAPI.get_match` and `PaladinsAPI.get_matches` methods,
     as well as from upgrading a `PartialMatch` object.
 
@@ -295,46 +338,50 @@ class Match(APIClient, MatchMixin):
     """
     def __init__(
         self,
-        api: "PaladinsAPI",
-        language: Language,
-        match_data: List[Dict[str, Any]],
+        api: DataCache,
+        cache_entry: Optional[CacheEntry],
+        match_data: List[responses.MatchPlayerObject],
         players: Dict[int, Player],
     ):
-        APIClient.__init__(self, api)
+        CacheClient.__init__(self, api)
         first_player = match_data[0]
         MatchMixin.__init__(self, first_player)
         logger.debug(f"Match(id={self.id}) -> creating...")
         self.replay_available: bool = first_player["hasReplay"] == "y"
         self.bans: List[Union[Champion, CacheObject]] = []
         for i in range(1, 5):
-            ban_id: int = first_player[f"BanId{i}"]
+            ban_id: int = first_player[f"BanId{i}"]  # type: ignore[misc]
+            # skip 0s
             if not ban_id:
-                # skip 0s
                 continue
-            ban_champ: Optional[Union[Champion, CacheObject]] = (
-                self._api.get_champion(ban_id, language)
-            )
+            ban_champ: Optional[Union[Champion, CacheObject]] = None
+            if cache_entry is not None:
+                ban_champ = cache_entry.champions.get(ban_id)
             if ban_champ is None:
-                ban_champ = CacheObject(id=ban_id, name=first_player[f"Ban_{i}"])
+                ban_champ = CacheObject(
+                    id=ban_id, name=first_player[f"Ban_{i}"]  # type: ignore[misc]
+                )
             self.bans.append(ban_champ)
         self.team1: List[MatchPlayer] = []
         self.team2: List[MatchPlayer] = []
+        # Determine party numbers
         # We need to do this here because apparently one-man parties are a thing
         party_count = count(1)
         parties: Dict[int, int] = {}
         for player_data in match_data:
             pid = player_data["PartyId"]
+            # process only non-0 parties
             if pid:
-                # process only non-0 parties
                 if pid not in parties:
                     # haven't seen this one yet, assign zero
                     parties[pid] = 0
                 elif parties[pid] == 0:
                     # we've seen this one, and it doesn't have a number assigned - assign one
                     parties[pid] = next(party_count)
+        # iterate over a second time, now that we have the party numbers sorted out
         for player_data in match_data:
-            match_player = MatchPlayer(self._api, language, player_data, parties, players)
-            team_number = player_data['TaskForce']
+            match_player = MatchPlayer(self, cache_entry, player_data, parties, players)
+            team_number = player_data["TaskForce"]
             if team_number == 1:
                 self.team1.append(match_player)
             elif team_number == 2:  # pragma: no branch
@@ -358,33 +405,40 @@ class Match(APIClient, MatchMixin):
 
         Uses up a single request to do the expansion.
         """
-        players = await self._api.get_players((p.player.id for p in self.players))
-        players_dict = {p.id: p for p in players}
+        players_dict = await _get_players(self._api, (p.player.id for p in self.players))
         for mp in self.players:
             pid = mp.player.id
-            if not pid:
-                # skip 0s
+            # skip 0s
+            if pid == 0:
                 continue
             if (p := players_dict.get(pid)) is not None:  # pragma: no branch
                 mp.player = p
 
 
-class LivePlayer(APIClient, WinLoseMixin):
+class LivePlayer(WinLoseMixin, CacheClient):
     """
     Represents a live match player.
     You can find these on the `LiveMatch.team1` and `LiveMatch.team2` attributes.
 
     Attributes
     ----------
+    match: LiveMatch
+        The match this player belongs to.
     player : Union[PartialPlayer, Player]
         The actual player playing in this match.
     champion : Union[Champion, CacheObject]
         The champion the player is using in this match.\n
         With incomplete cache, this will be a `CacheObject` with the name and ID set.
-    skin : CacheObject
-        The skin the player has equipped for this match.
-    rank : Rank
+    skin : Union[Skin, CacheObject]
+        The skin the player has equipped for this match.\n
+        With incomplete cache, this will be a `CacheObject` with the name and ID set.
+    rank : Optional[Rank]
         The player's rank.
+
+        .. warning::
+
+            Due to API limitations, this is only available for matches played in ranked queues.\n
+            For other queues, this attribute will be `None`.
     account_level : int
         The player's account level.
     mastery_level : int
@@ -396,34 +450,52 @@ class LivePlayer(APIClient, WinLoseMixin):
     """
     def __init__(
         self,
-        api: "PaladinsAPI",
-        language: Language,
-        player_data: Dict[str, Any],
+        match: LiveMatch,
+        cache_entry: Optional[CacheEntry],
+        player_data: responses.LivePlayerObject,
         players: Dict[int, Player],
     ):
-        APIClient.__init__(self, api)
+        CacheClient.__init__(self, match._api)
         WinLoseMixin.__init__(
             self,
             wins=player_data["tierWins"],
             losses=player_data["tierLosses"],
         )
+        self.match: LiveMatch = match
+        # Player
         player: Optional[Union[PartialPlayer, Player]] = players.get(int(player_data["playerId"]))
         if player is None:
             # if no full player was found
-            from .player import PartialPlayer  # noqa, cyclic imports
+            from .player import PartialPlayer  # cyclic imports
             player = PartialPlayer(
-                api, id=player_data["playerId"], name=player_data["playerName"]
+                self._api,
+                id=player_data["playerId"],
+                name=player_data["playerName"],
+                platform=player_data["playerPortalId"],
             )
         self.player: Union[PartialPlayer, Player] = player
+        # Champion
         champion_id: int = player_data["ChampionId"]
-        champion: Optional[Union[Champion, CacheObject]] = (
-            self._api.get_champion(champion_id, language)
-        )
+        champion: Optional[Union[Champion, CacheObject]] = None
+        if cache_entry is not None:
+            champion = cache_entry.champions.get(champion_id)
         if champion is None:
             champion = CacheObject(id=champion_id, name=player_data["ChampionName"])
         self.champion: Union[Champion, CacheObject] = champion
-        self.skin = CacheObject(id=player_data["SkinId"], name=player_data["Skin"])
-        self.rank = Rank(player_data["Tier"], return_default=True)
+        # Skin
+        skin_id = player_data["SkinId"]
+        skin: Optional[Union[Skin, CacheObject]] = None
+        if cache_entry is not None:
+            skin = cache_entry.skins.get(skin_id)
+        if skin is None:  # pragma: no cover
+            skin = CacheObject(id=skin_id, name=player_data["Skin"])
+        self.skin: Union[Skin, CacheObject] = skin
+        # Other
+        self.rank: Optional[Rank]
+        if match.queue.is_ranked():  # pragma: no cover
+            self.rank = Rank(player_data["Tier"], _return_default=True)
+        else:
+            self.rank = None
         self.account_level: int = player_data["Account_Level"]
         self.mastery_level: int = player_data["Mastery_Level"]
 
@@ -435,7 +507,7 @@ class LivePlayer(APIClient, WinLoseMixin):
         )
 
 
-class LiveMatch(APIClient):
+class LiveMatch(CacheClient):
     """
     Represents an on-going live match.
     You can get this from the `PlayerStatus.get_live_match` method.
@@ -459,21 +531,21 @@ class LiveMatch(APIClient):
     """
     def __init__(
         self,
-        api: "PaladinsAPI",
-        language: Language,
-        match_data: List[Dict[str, Any]],
+        api: DataCache,
+        cache_entry: Optional[CacheEntry],
+        match_data: List[responses.LivePlayerObject],
         players: Dict[int, Player],
     ):
         super().__init__(api)
         first_player = match_data[0]
         self.id: int = first_player["Match"]
         self.map_name: str = _convert_map_name(first_player["mapGame"])
-        self.queue = Queue(int(first_player["Queue"]), return_default=True)
-        self.region = Region(first_player["playerRegion"], return_default=True)
+        self.queue = Queue(int(first_player["Queue"]), _return_default=True)
+        self.region = Region(first_player["playerRegion"], _return_default=True)
         self.team1: List[LivePlayer] = []
         self.team2: List[LivePlayer] = []
         for player_data in match_data:
-            live_player = LivePlayer(self._api, language, player_data, players)
+            live_player = LivePlayer(self, cache_entry, player_data, players)
             if player_data["taskForce"] == 1:
                 self.team1.append(live_player)
             elif player_data["taskForce"] == 2:  # pragma: no branch
@@ -496,12 +568,11 @@ class LiveMatch(APIClient):
 
         Uses up a single request to do the expansion.
         """
-        players = await self._api.get_players((p.player.id for p in self.players))
-        players_dict = {p.id: p for p in players}
+        players_dict = await _get_players(self._api, (p.player.id for p in self.players))
         for mp in self.players:
             pid = mp.player.id
-            if not pid:
-                # skip 0s
+            # skip 0s
+            if pid == 0:  # pragma: no cover
                 continue
             if (p := players_dict.get(pid)) is not None:  # pragma: no branch
                 mp.player = p
