@@ -1,10 +1,13 @@
 from __future__ import annotations
 
+import sys
 from math import floor
+from collections import OrderedDict
+from difflib import SequenceMatcher
 from functools import partialmethod
 from weakref import WeakValueDictionary
 from datetime import datetime, timedelta
-from operator import attrgetter, eq, ne, lt, le, gt, ge
+from operator import itemgetter, attrgetter, eq, ne, lt, le, gt, ge
 from typing import (
     Optional,
     Union,
@@ -16,20 +19,23 @@ from typing import (
     Callable,
     Iterable,
     Iterator,
+    Sequence,
     Generator,
     AsyncGenerator,
     TypeVar,
+    Literal,
     cast,
     overload,
 )
 
-from .mixins import CacheObject
+from .mixins import CacheObject as CacheObject_, Expandable
 
 
 __all__ = [
     # functions
     "get",
     "chunk",
+    "group_by",
     "expand_partial",
     # classes
     "Lookup",
@@ -37,14 +43,40 @@ __all__ = [
     "WeakValueDefaultDict",
 ]
 # Type variable for internal utils typing
-X = TypeVar("X")
-Y = TypeVar("Y")
+_X = TypeVar("_X")
+_Y = TypeVar("_Y")
 
 
-LookupType = TypeVar("LookupType", bound=CacheObject)
+CacheObject = TypeVar("CacheObject", bound=CacheObject_)
 
 
-def _convert_timestamp(timestamp: str) -> Optional[datetime]:
+def _deduplicate(iterable: Iterable[_X], *to_remove: _X) -> List[_X]:
+    """
+    Removes duplicates from an iterable and returns a list. Optimised for speed.
+    Optionally, also removes the value(s) specified entirely.
+
+    Parameters
+    ----------
+    iterable : Iterable[X]
+        The iterable of values to deduplicate.
+    *to_remove: X
+        Optional value(s) to remove.
+
+    Returns
+    -------
+    List[X]
+        The deduplicated list of values.
+    """
+    if not isinstance(iterable, Iterable):
+        raise TypeError(f"Expected an iterable, got {type(iterable)}")
+    no_dups: List[_X] = list(OrderedDict.fromkeys(iterable))
+    for value in to_remove:
+        if value in no_dups:
+            no_dups.remove(value)
+    return no_dups
+
+
+def _convert_timestamp(timestamp: str) -> datetime:
     """
     Converts the timestamp format returned by the API.
 
@@ -55,13 +87,10 @@ def _convert_timestamp(timestamp: str) -> Optional[datetime]:
 
     Returns
     -------
-    Optional[datetime]
-        A converted datetime object.\n
-        `None` is returned if an empty string was passed.
+    datetime
+        A converted datetime object.
     """
-    if timestamp:
-        return datetime.strptime(timestamp, "%m/%d/%Y %I:%M:%S %p")
-    return None
+    return datetime.strptime(timestamp, "%m/%d/%Y %I:%M:%S %p")
 
 
 def _convert_map_name(map_name: str) -> str:
@@ -79,38 +108,45 @@ def _convert_map_name(map_name: str) -> str:
         The converted map name.
     """
     map_name = map_name.strip()
-    for prefix in ("LIVE ", "Ranked ", "Practice ", "WIP "):
+    for prefix in ("LIVE", "Ranked", "Practice", "WIP"):  # pragma: no branch
         if map_name.startswith(prefix):
             map_name = map_name[len(prefix):]
-    for suffix in (" (Siege)", " (Onslaught)", " (TDM)", " (KOTH)"):
+            break
+    for suffix in ("(Siege)", "(Onslaught)", "(TDM)", "(KOTH)"):
         if map_name.endswith(suffix):
             map_name = map_name[:-len(suffix)]
-    return map_name
+            break
+    return map_name.strip()
+
+
+def _floor_dt(dt: datetime, td: timedelta) -> datetime:
+    return dt - (dt - datetime.min) % td
+
+
+def _ceil_dt(dt: datetime, td: timedelta) -> datetime:
+    return dt + (datetime.min - dt) % td
 
 
 # Generates API-valid series of date and hour parameters for the 'getmatchidsbyqueue' endpoint
 def _date_gen(
     start: datetime, end: datetime, *, reverse: bool = False
 ) -> Generator[Tuple[str, str], None, None]:
-    # normalize and floor start and end to 10 minutes step resolution
-    start = start.replace(minute=(
-        start.minute // 10 * 10
-    ), second=0, microsecond=0)
-    end = end.replace(minute=(
-        end.minute // 10 * 10
-    ), second=0, microsecond=0)
-    # check if the time slice is too short - save on processing by quitting early
-    if start >= end:
-        return
     # helpful time intervals
     one_day = timedelta(days=1)
     one_hour = timedelta(hours=1)
     ten_minutes = timedelta(minutes=10)
+    # round start and end to the nearest multiply of 10m
+    # floor start and ceil end
+    start = _floor_dt(start, ten_minutes)
+    end = _ceil_dt(end, ten_minutes)
+    # check if the time slice is too short - save on processing by quitting early
+    if start >= end:
+        return
 
     if reverse:
         if end.minute > 0:
             # round down end to the nearest hour
-            closest_hour = end.replace(minute=0)
+            closest_hour = _floor_dt(end, one_hour)
             while end > closest_hour:
                 end -= ten_minutes
                 yield (end.strftime("%Y%m%d"), f"{end.hour},{end.minute:02}")
@@ -118,7 +154,7 @@ def _date_gen(
                     return
         if end.hour > 0:
             # round down end to the nearest day midnight
-            closest_day = end.replace(hour=0, minute=0)
+            closest_day = _floor_dt(end, one_day)
             if closest_day >= start:
                 while end > closest_day:
                     end -= one_hour
@@ -126,10 +162,7 @@ def _date_gen(
                     if end <= start:
                         return
         # round up start to the nearest end day midnight
-        closest_day = start.replace(hour=0, minute=0)
-        if closest_day != start:
-            # only add one day if hours or minutes differ
-            closest_day += one_day
+        closest_day = _ceil_dt(start, one_day)
         while end > closest_day:
             end -= one_day
             yield (end.strftime("%Y%m%d"), "-1")
@@ -137,10 +170,7 @@ def _date_gen(
             return
         if start.hour > 0:
             # round up start to the nearest hour
-            closest_hour = start.replace(minute=0)
-            if closest_hour != start:
-                # only add one hour if minutes differ
-                closest_hour += one_hour
+            closest_hour = _ceil_dt(start, one_hour)
             while end > closest_hour:
                 end -= one_hour
                 yield (end.strftime("%Y%m%d"), str(end.hour))
@@ -153,7 +183,7 @@ def _date_gen(
     else:
         if start.minute > 0:
             # round up start to the nearest hour
-            closest_hour = start.replace(minute=0) + one_hour
+            closest_hour = _ceil_dt(start, one_hour)
             while start < closest_hour:
                 yield (start.strftime("%Y%m%d"), f"{start.hour},{start.minute:02}")
                 start += ten_minutes
@@ -161,7 +191,7 @@ def _date_gen(
                     return
         if start.hour > 0:
             # round up start to the nearest day midnight
-            closest_day = start.replace(hour=0, minute=0) + one_day
+            closest_day = _ceil_dt(start, one_day)
             if closest_day <= end:
                 while start < closest_day:
                     yield (start.strftime("%Y%m%d"), str(start.hour))
@@ -169,7 +199,7 @@ def _date_gen(
                     if start >= end:
                         return
         # round down end to the nearest end day midnight
-        closest_day = end.replace(hour=0, minute=0)
+        closest_day = _floor_dt(end, one_day)
         while start < closest_day:
             yield (start.strftime("%Y%m%d"), "-1")
             start += one_day
@@ -177,7 +207,7 @@ def _date_gen(
             return
         if end.hour > 0:
             # round down end to the nearest end hour
-            closest_hour = end.replace(minute=0)
+            closest_hour = _floor_dt(end, one_hour)
             while start < closest_hour:
                 yield (start.strftime("%Y%m%d"), str(start.hour))
                 start += one_hour
@@ -189,7 +219,7 @@ def _date_gen(
             start += ten_minutes
 
 
-def get(iterable: Iterable[X], **attrs) -> Optional[X]:
+def get(iterable: Iterable[_X], **attrs) -> Optional[_X]:
     """
     Returns the first object from the ``iterable`` which attributes match the
     keyword arguments passed.
@@ -226,73 +256,226 @@ def get(iterable: Iterable[X], **attrs) -> Optional[X]:
     return None
 
 
-class Lookup(Iterable[LookupType]):
+def group_by(iterable: Iterable[_X], key: Callable[[_X], _Y]) -> Dict[_Y, List[_X]]:
     """
-    A helper class utilizing an internal list and three dictionaries, allowing for easy indexing
-    and lookup based on Name and ID attributes. Supports fuzzy Name searches too.
+    A helper function for grouping elements of an iterable into a dictionary, where each key
+    represents a common value, and the value represents a list of elements having said
+    common value.
 
-    This object resembles an immutable list, but exposes `__len__` and `__iter__` special
-    methods for ease of use.
+    Parameters
+    ----------
+    iterable : Iterable[X]
+        An iterable of elements to group.
+    key : Callable[[X], Y]
+        A function that takes each element from the provided iterable as it's parameter,
+        and outputs a group to which said element belongs to.
+
+    Returns
+    -------
+    Dict[Y, List[X]]
+        A mapping of groups to lists of grouped elements.
     """
-    def __init__(self, iterable: Iterable[LookupType]):
-        self._list_lookup: List[LookupType] = []
-        self._id_lookup: Dict[int, LookupType] = {}
-        self._name_lookup: Dict[str, LookupType] = {}
-        self._fuzzy_lookup: Dict[str, LookupType] = {}
+    item_map: Dict[_Y, List[_X]] = {}
+    for item in iterable:
+        group = key(item)
+        if group not in item_map:
+            item_map[group] = []
+        item_map[group].append(item)
+    return item_map
+
+
+class Lookup(Sequence[CacheObject]):
+    """
+    A helper class utilizing an internal list and two dictionaries, allowing for easy indexing
+    and lookup of `CacheObject` and it's subclasses, based on the Name and ID attributes.
+    Supports fuzzy Name searches too.
+
+    This object resembles an immutable list, and thus exposes ``__len__``, ``__iter__``,
+    ``__getitem__``, ``__contains__``, ``__reversed__``, ``index`` and ``count``
+    special methods for ease of use.
+    """
+    def __init__(self, iterable: Iterable[CacheObject]):
+        self._list_lookup: List[CacheObject] = []
+        self._id_lookup: Dict[int, CacheObject] = {}
+        self._name_lookup: Dict[str, CacheObject] = {}
         for e in iterable:
             self._list_lookup.append(e)
             self._id_lookup[e.id] = e
-            self._name_lookup[e.name] = e
-            self._fuzzy_lookup[e.name.lower()] = e
+            self._name_lookup[e.name.lower()] = e
 
     def __repr__(self) -> str:
-        return f"{self.__class__.__name__}({self._list_lookup.__repr__()})"
+        return f"{self.__class__.__name__}({repr(self._list_lookup)})"
 
     def __len__(self) -> int:
-        """
-        Returns the length of the internal list.\n
-        Use ``len()`` on this object to obtain it.
-
-        :type: int
-        """
         return len(self._list_lookup)
 
-    def __iter__(self) -> Iterator[LookupType]:
-        """
-        Returns an iterator over the internal list.\n
-        Use ``iter()`` on this object to obtain it.
-
-        :type: Iterator[LookupType]
-        """
+    def __iter__(self) -> Iterator[CacheObject]:
         return iter(self._list_lookup)
 
-    def _lookup(self, name_or_id: Union[int, str], *, fuzzy: bool = False) -> Optional[LookupType]:
+    @overload
+    def __getitem__(self, index: int) -> CacheObject:
+        ...
+
+    @overload
+    def __getitem__(self, index: slice) -> List[CacheObject]:
+        ...
+
+    def __getitem__(self, index: Union[int, slice]) -> Union[CacheObject, List[CacheObject]]:
+        return self._list_lookup[index]
+
+    def __contains__(self, item: object) -> bool:
+        return item in self._list_lookup
+
+    def __reversed__(self) -> Iterator[CacheObject]:
+        return reversed(self._list_lookup)
+
+    def index(self, item: CacheObject, start: int = 0, stop: int = sys.maxsize) -> int:
+        return self._list_lookup.index(item, start, stop)
+
+    def count(self, item: CacheObject) -> int:
+        return self._list_lookup.count(item)
+
+    def get(self, name_or_id: Union[int, str]) -> Optional[CacheObject]:
         """
         Allows you to quickly lookup an element by it's Name or ID.
 
         Parameters
         ----------
         name_or_id : Union[int, str]
-            Name or ID of the element you want to lookup.
-        fuzzy : bool
-            When set to `True`, makes the Name search case insensitive.\n
+            The name or ID of the element you want to lookup.
+
+            .. note::
+
+                The name lookup is case-insensitive.
+
+        Returns
+        -------
+        Optional[CacheObject]
+            The element requested.\n
+            `None` is returned if the requested element couldn't be found.
+        """
+        if isinstance(name_or_id, str):
+            return self._name_lookup.get(name_or_id.lower())
+        return self._id_lookup.get(name_or_id)
+
+    @overload
+    def get_fuzzy_matches(
+        self,
+        name: str,
+        *,
+        limit: int = 3,
+        cutoff: float = 0.6,
+        with_scores: Literal[False] = False,
+    ) -> List[CacheObject]:
+        ...
+
+    @overload
+    def get_fuzzy_matches(
+        self, name: str, *, limit: int = 3, cutoff: float = 0.6, with_scores: Literal[True]
+    ) -> List[Tuple[CacheObject, float]]:
+        ...
+
+    def get_fuzzy_matches(
+        self, name: str, *, limit: int = 3, cutoff: float = 0.6, with_scores: bool = False
+    ) -> Union[List[CacheObject], List[Tuple[CacheObject, float]]]:
+        """
+        Performs a fuzzy lookup of an element by it's name,
+        by calculating the similarity score between each item. Case-insensitive.\n
+        See also: `get_fuzzy`.
+
+        Parameters
+        ----------
+        name : str
+            The name of the element you want to lookup.
+        limit : int
+            The maximum amount of elements to return in the list. Has to be greater than ``0``.\n
+            Defaults to ``3``.
+        cutoff : float
+            The similarity score cutoff range, below which matches will be excluded
+            from the output. Lower values have a better chance of yielding correct results,
+            but also a higher chance of false-positives. Accepted range is 0 to 1.\n
+            Defaults to ``0.6``.
+        with_scores : bool
+            If set to `True`, returns a list of 2-item tuples, with the similar element
+            as the first item, and its score as the second.\n
             Defaults to `False`.
 
         Returns
         -------
-        Optional[LookupType]
+        Union[List[CacheObject], List[Tuple[CacheObject, float]]]
+            A list of up to ``limit`` matching elements, with at least ``cutoff`` similarity score,
+            sorted in descending order by their similarity score.\n
+            If ``with_scores`` is set to `True`, returns a list of up to ``limit`` 2-item tuples,
+            where the first item of each tuple is the element, and the second item
+            is the similarity score it has.
+
+        Raises
+        ------
+        TypeError
+            ``name``, ``limit`` or ``cutoff`` arguments are of incorrect type
+        ValueError
+            ``limit`` or ``cutoff`` arguments have an incorrect value
+        """
+        if not isinstance(name, str):
+            raise TypeError("The name has to be a string of characters")
+        if not isinstance(limit, int):
+            raise TypeError("limit has to be a positive non-zero integer")
+        if not isinstance(cutoff, float):
+            raise TypeError("cutoff has to be a float in 0-1 range")
+        if not limit > 0:
+            raise ValueError("limit has to be a positive non-zero integer")
+        if not 0 <= cutoff <= 1:
+            raise ValueError("cutoff has to be a float in 0-1 range")
+
+        matcher: SequenceMatcher[str] = SequenceMatcher()
+        matcher.set_seq2(name.lower())
+        scores: List[Tuple[str, float]] = []
+        for key in self._name_lookup:
+            matcher.set_seq1(key)
+            if (
+                matcher.real_quick_ratio() >= cutoff
+                and matcher.quick_ratio() >= cutoff
+                and (score := matcher.ratio()) >= cutoff
+            ):
+                scores.append((key, score))
+        scores.sort(key=itemgetter(1), reverse=True)
+        if with_scores:
+            return [(self._name_lookup[key], score) for key, score in scores[:limit]]
+        return [self._name_lookup[key] for key, score in scores[:limit]]
+
+    def get_fuzzy(self, name: str, *, cutoff: float = 0.6) -> Optional[CacheObject]:
+        """
+        Simplified version of `get_fuzzy_matches`, allowing you to search for a single element,
+        or receive `None` if no matching element was found.
+
+        Parameters
+        ----------
+        name : str
+            The name of the element you want to lookup.
+        cutoff : float, optional
+            The similarity score cutoff range. See: `get_fuzzy_matches` for more information.\n
+            Defaults to ``0.6``.
+
+        Returns
+        -------
+        Optional[CacheObject]
             The element requested.\n
             `None` is returned if the requested element couldn't be found.
+
+        Raises
+        ------
+        TypeError
+            ``name`` or ``cutoff`` arguments are of incorrect type
+        ValueError
+            ``cutoff`` argument has an incorrect value
         """
-        if isinstance(name_or_id, int):
-            return self._id_lookup.get(name_or_id)
-        if fuzzy and isinstance(name_or_id, str):
-            name_or_id = name_or_id.lower()
-            return self._fuzzy_lookup.get(name_or_id)
-        return self._name_lookup.get(name_or_id)
+        matches = self.get_fuzzy_matches(name, limit=1, cutoff=cutoff)
+        if matches:
+            return matches[0]
+        return None
 
 
-def chunk(list_to_chunk: List[X], chunk_length: int) -> Generator[List[X], None, None]:
+def chunk(list_to_chunk: List[_X], chunk_length: int) -> Generator[List[_X], None, None]:
     """
     A helper generator that divides the input list into chunks of ``chunk_length`` length.
     The last chunk may be shorter than specified.
@@ -308,7 +491,7 @@ def chunk(list_to_chunk: List[X], chunk_length: int) -> Generator[List[X], None,
         yield list_to_chunk[i:i + chunk_length]
 
 
-async def expand_partial(iterable: Iterable) -> AsyncGenerator:
+async def expand_partial(iterable: Iterable[_X]) -> AsyncGenerator[_X, None]:
     """
     A helper async generator that can be used to automatically expand partial objects for you.
     Any other object found in the ``iterable`` is passed unchanged.
@@ -327,10 +510,8 @@ async def expand_partial(iterable: Iterable) -> AsyncGenerator:
     AsyncGenerator
         An async generator yielding expanded versions of each partial object.
     """
-    from .player import PartialPlayer  # cyclic imports
-    from .match import PartialMatch  # cyclic imports
     for element in iterable:
-        if isinstance(element, (PartialPlayer, PartialMatch)):
+        if isinstance(element, Expandable):
             expanded = await element
             yield expanded
         else:
@@ -362,8 +543,8 @@ class Duration:
     def __init__(self, **kwargs):
         self._delta = timedelta(**kwargs)
         self._total_seconds = self._delta.total_seconds()
-        seconds, ms_fraction = divmod(self._total_seconds, 1)
-        self._microseconds = round(ms_fraction * 1e6)  # convert the fractional seconds
+        seconds, us_fraction = divmod(self._total_seconds, 1)
+        self._microseconds = round(us_fraction * 1e6)  # convert the fractional seconds
         minutes, seconds = _int_divmod(seconds, 60)
         self._seconds = seconds
         hours, minutes = _int_divmod(minutes, 60)
@@ -473,7 +654,7 @@ class Duration:
             ms = ''
         return f"{days}{hours}{self._minutes:02}:{self._seconds:02}{ms}"
 
-    def _get_delta(self, other: object) -> Union[timedelta, NotImplemented]:
+    def _get_delta(self, other: object) -> timedelta:
         if isinstance(other, Duration):
             return other._delta
         elif isinstance(other, timedelta):
@@ -482,18 +663,13 @@ class Duration:
 
     # Comparisons
 
-    def _cmp(
-        self, opr: Callable[[object, object], bool], other: object, *, base: bool = False
-    ) -> bool:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
-            if base:
-                return opr is ne
+    def _cmp(self, opr: Callable[[object, object], bool], other: object) -> bool:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return opr(self._delta, delta)
 
-    __eq__ = cast(Callable[[object, object], bool], partialmethod(_cmp, eq, base=True))
-    __ne__ = cast(Callable[[object, object], bool], partialmethod(_cmp, ne, base=True))
+    __eq__ = cast(Callable[[object, object], bool], partialmethod(_cmp, eq))
+    __ne__ = cast(Callable[[object, object], bool], partialmethod(_cmp, ne))
     __lt__ = partialmethod(_cmp, lt)
     __le__ = partialmethod(_cmp, le)
     __gt__ = partialmethod(_cmp, gt)
@@ -502,22 +678,19 @@ class Duration:
     # Math operations
 
     def __add__(self, other: Union[Duration, timedelta]) -> Duration:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return Duration(seconds=self._total_seconds + delta.total_seconds())
 
     __radd__ = __add__
 
     def __sub__(self, other: Union[Duration, timedelta]) -> Duration:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return Duration(seconds=self._total_seconds - delta.total_seconds())
 
     def __rsub__(self, other: Union[Duration, timedelta]) -> Duration:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return Duration(seconds=delta.total_seconds() - self._total_seconds)
 
@@ -539,8 +712,7 @@ class Duration:
     def __truediv__(self, other: Union[Duration, timedelta, int, float]):
         if isinstance(other, (int, float)):
             return Duration(seconds=self._total_seconds / other)
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return self._total_seconds / delta.total_seconds()
 
@@ -560,8 +732,7 @@ class Duration:
     def __floordiv__(self, other: Union[Duration, timedelta, int]):
         if isinstance(other, int):
             return Duration(microseconds=floor(self._total_seconds * 1e6 // other))
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return int(self._total_seconds // delta.total_seconds())
 
@@ -571,20 +742,17 @@ class Duration:
         return int(other.total_seconds() // self._total_seconds)
 
     def __mod__(self, other: Union[Duration, timedelta]) -> Duration:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return Duration(seconds=(self._total_seconds % delta.total_seconds()))
 
     def __rmod__(self, other: Union[Duration, timedelta]) -> Duration:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         return Duration(seconds=(delta.total_seconds() % self._total_seconds))
 
     def __divmod__(self, other: Union[Duration, timedelta]) -> Tuple[int, Duration]:
-        delta = self._get_delta(other)
-        if delta is NotImplemented:
+        if (delta := self._get_delta(other)) is NotImplemented:
             return NotImplemented
         q, r = divmod(self._total_seconds, delta.total_seconds())
         return (int(q), Duration(seconds=r))
@@ -607,16 +775,16 @@ class Duration:
         return Duration(seconds=self._total_seconds)
 
 
-class WeakValueDefaultDict(WeakValueDictionary, Mapping[X, Y]):
+class WeakValueDefaultDict(WeakValueDictionary, Mapping[_X, _Y]):  # type: ignore[type-arg]
     def __init__(
         self,
         default_factory: Optional[Callable[[], Any]] = None,
-        mapping_or_iterable: Union[Mapping[X, Y], Iterable[Tuple[X, Y]]] = {},
+        mapping_or_iterable: Union[Mapping[_X, _Y], Iterable[Tuple[_X, _Y]]] = {},
     ):
         self.default_factory = default_factory
         super().__init__(mapping_or_iterable)
 
-    def __getitem__(self, key: X) -> Y:
+    def __getitem__(self, key: _X) -> _Y:
         try:
             return super().__getitem__(key)
         except KeyError:
